/* -*- mode: js; js-basic-offset: 4; indent-tabs-mode: nil -*- */

/*
    Copyright (C) 2013  Borsato Ivano

    The JavaScript code in this page is free software: you can
    redistribute it and/or modify it under the terms of the GNU
    General Public License (GNU GPL) as published by the Free Software
    Foundation, either version 3 of the License, or (at your option)
    any later version.  The code is distributed WITHOUT ANY WARRANTY;
    without even the implied warranty of MERCHANTABILITY or FITNESS
    FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

    As additional permission under GNU GPL version 3 section 7, you
    may distribute non-source (e.g., minimized or compacted) forms of
    that code without the copy of the GNU GPL normally required by
    section 4, provided you include this license notice and a URL
    through which recipients can access the Corresponding Source.
*/

const Gio = imports.gi.Gio;
const St = imports.gi.St;
const Meta = imports.gi.Meta;
const Shell = imports.gi.Shell;
const Lang = imports.lang;
const PanelMenu = imports.ui.panelMenu;
const Panel = imports.ui.panel;
const PopupMenu = imports.ui.popupMenu;
const MessageTray = imports.ui.messageTray;
const Main = imports.ui.main;
const LibRecorder = imports.ui.components.recorder;


const Gettext = imports.gettext.domain('EasyScreenCast@iacopodeenosee.gmail.com');
const _ = Gettext.gettext;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Lib = Me.imports.convenience;
const Time = Me.imports.timer;

// setting keys - own
const SHOW_TIMER_REC_SETTING_KEY = "show-timer-rec";

let Indicator;
let timerD,timerC=null;
let ScreenVideo = new LibRecorder.Recorder();
let isActive = false;


const EasyScreenCast_Indicator = new Lang.Class({
    Name: 'EasyScreenCast.indicator',
    Extends: PanelMenu.SystemStatusButton,

    _init: function() {
	    this.parent(null, "EasyScreenCast-indicator"); 
	    //prepare setting var
	    this.settings = Lib.getSettings("org.gnome.shell.extensions.EasyScreenCast");
	    this.isDelayActive=this.settings.get_boolean('active-delay-time');
	    this.isShowNotify=this.settings.get_boolean(SHOW_TIMER_REC_SETTING_KEY);
	    
	    //add icon
        this.setGIcon(Lib.ESCoffGIcon);
        
        this.DelayTimeSlider=null;
        this.notifyCounting;
        
        //add start/stop menu entry
	    this.imRecordAction = new PopupMenu.PopupBaseMenuItem();
	    this.RecordingLabel = new St.Label({ text: _("Start recording"),
	                                        style_class: 'RecordAction-label' });
	    this.imRecordAction.addActor(this.RecordingLabel, 
	                                { align: St.Align.START });
	    this.menu.addMenuItem(this.imRecordAction);
	    this.imRecordAction.connect('activate', Lang.bind(this, function(){
            this.isShowNotify=this.settings.get_boolean(SHOW_TIMER_REC_SETTING_KEY);
            this._doDelayAction();
        }));
	    
        //add separetor menu
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());

        //add delay menu entry
	    this.imDelayRec = new PopupMenu.PopupSwitchMenuItem(_("Delay recording"),
	                        this.settings.get_boolean('active-delay-time'),
                            { style_class: 'popup-subtitle-menu-item' });
	    this.imDelayRec.connect("toggled", Lang.bind(this, function(item){
            if (item.state) {
			    this.isDelayActive=true;
		    } else {
			    this.isDelayActive=false;
		    }
		    this.settings.set_boolean('active-delay-time', item.state);
        }));
        
	    this.menu.addMenuItem(this.imDelayRec);

	    
	    this.DelayTimeTitle = new PopupMenu.PopupMenuItem(_("Delay Time"),
	                             { reactive: false });
        this.DelayTimeLabel = new St.Label({ text: Math.floor(this.settings.get_int('delay-time')).toString()
                                                    +_(" Sec") });
        this.DelayTimeSlider = new PopupMenu.PopupSliderMenuItem(this.settings.get_int('delay-time')/100);
        this.DelayTimeSlider.connect('value-changed', Lang.bind(this, function(item) {
            this.DelayTimeLabel.set_text(Math.floor(item.value*100).toString()+_(" Sec"));
        }));
        
        this.DelayTimeSlider.connect('drag-end', Lang.bind(this, this._onDelayTimeChanged));
        this.DelayTimeSlider.actor.connect('scroll-event', 
                                Lang.bind(this, this._onDelayTimeChanged));
        this.DelayTimeTitle.addActor(this.DelayTimeLabel, { align: St.Align.END });
        this.menu.addMenuItem(this.DelayTimeTitle);
        this.menu.addMenuItem(this.DelayTimeSlider);


        //add separetor menu
        this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
        //add option menu entry
	    this.imOptions = new PopupMenu.PopupMenuItem(_("Options"));
	    this.menu.addMenuItem(this.imOptions);
	    this.imOptions.connect('activate', Lang.bind(this, this._doExtensionPreferences));
	   
    },
    
    _doDelayAction: function() {
        if(this.isDelayActive  && !isActive){
            global.log('ESC > delay recording called | delay= ' + this.DelayTimeSlider.value);
       	    timerD = new Time.TimerDelay((Math.floor(this.DelayTimeSlider.value*100)),this._doRecording,this);
       	    timerD.begin();
        } else {
            global.log('ESC > instant recording called');
            this._doRecording();
        }
    },

	_doRecording: function() {
        
        //start/stop record screen
        ScreenVideo._toggleRecorder();
        
        if(isActive===false){
            global.log('ESC > start recording');
            isActive=true;

            if(this.isShowNotify){
                global.log('ESC > show notify');
                //create notify
                var source = new MessageTray.SystemNotificationSource();
                
                this.notifyCounting  = new MessageTray.Notification(source, 
                                        _("Start Recording"),
                                        null,
                                        { gicon: Lib.ESConGIcon });
                this.notifyCounting.setTransient(false);
                this.notifyCounting.setResident(true);
        
                Main.messageTray.add(source);
                source.notify(this.notifyCounting);
                
                //start counting rec
                timerC = new Time.TimerCounting(refreshNotify,this);
                timerC.begin();
            }
        } else {
            global.log('ESC > stop recording');
            isActive=false;
            
            if(timerC!==null){
                //stop counting rec
                timerC.halt();
                timerC=null;
            }

        }
        
        refreshIndicator();
	},

	_doExtensionPreferences: function() {
	    global.log('ESC > open preferences');
		Main.Util.trySpawnCommandLine('gnome-shell-extension-prefs EasyScreenCast@iacopodeenosee.gmail.com');
	},

    _onDelayTimeChanged: function() {
        this.settings.set_int('delay-time', Math.floor(this.DelayTimeSlider.value*100));
    },

    destroy: function() {
		this.parent();
    }
});

function refreshIndicator() {
    global.log('ESC > refresh indicator');
    if(Indicator!==null){
        if(isActive===true){
            Indicator.setGIcon(Lib.ESConGIcon);
            Indicator.RecordingLabel.set_text(_("Stop recording"));
        } else {
            Indicator.setGIcon(Lib.ESCoffGIcon);
            Indicator.RecordingLabel.set_text(_("Start recording"));
        }
    }
}

function refreshNotify(sec,alertEnd){
    if(Indicator.notifyCounting!==null || Indicator.notifyCounting === undefined){
        if(alertEnd){
            Indicator.notifyCounting.update(_("EasyScreenCast -> Finish Recording"),
                                            _("total Seconds : "+ sec),
                                            { gicon: Lib.ESCoffGIcon });
        } else {
            Indicator.notifyCounting.update(_("EasyScreenCast -> Seconds passed : ") + sec,
                                            _("Recording in progress"),
                                            { gicon: Lib.ESConGIcon });
        }
    }
}




function init(meta) {
	global.log('ESC > initExtension called');
	
	Lib.initTranslations('EasyScreenCast@iacopodeenosee.gmail.com');
}


function enable() {
	global.log('ESC > enableExtension called');
	
    Indicator = new EasyScreenCast_Indicator;
    Main.panel.addToStatusArea('EasyScreenCast-indicator', Indicator);
    
    //binding keys
    var BindKeys = new Gio.Settings({ schema: 'org.gnome.shell.keybindings' });
    Main.wm.addKeybinding('toggle-recording',
        BindKeys,
        Meta.KeyBindingFlags.NONE,
        Shell.KeyBindingMode.ALL,
        Lang.bind(this, refreshIndicator));
}

function disable() {
	global.log('ESC > disableExtension called');
    
    if(timerD!==nul || timerD!==undefined){
        timerD.stop();
    }
    
    Main.wm.removeKeybinding('toggle-recording');
    Indicator.destroy();
}
